@@ -32,11 +32,17 @@
 
 /*** DeviceWidget ***/
 DeviceWidget::DeviceWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& x) :
-    MinimalStreamWidget(cobject, x),
+    MinimalStreamWidget(cobject),
     offsetButtonEnabled(false) {
 
-    x->get_widget("lockToggleButton", lockToggleButton);
-    x->get_widget("muteToggleButton", muteToggleButton);
+    /* MinimalStreamWidget member variables. */
+    x->get_widget("deviceChannelsVBox", channelsVBox);
+    x->get_widget("deviceNameLabel", nameLabel);
+    x->get_widget("deviceBoldNameLabel", boldNameLabel);
+    x->get_widget("deviceIconImage", iconImage);
+
+    x->get_widget("deviceLockToggleButton", lockToggleButton);
+    x->get_widget("deviceMuteToggleButton", muteToggleButton);
     x->get_widget("defaultToggleButton", defaultToggleButton);
     x->get_widget("portSelect", portSelect);
     x->get_widget("portList", portList);
@@ -64,35 +70,26 @@ DeviceWidget::DeviceWidget(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Buil
     for (unsigned i = 0; i < PA_CHANNELS_MAX; i++)
         channelWidgets[i] = NULL;
 
-#ifdef HAVE_GTK3
-    offsetAdjustment = Gtk::Adjustment::create(0.0, -2000.0, 2000.0, 10.0, 50.0, 0.0);
+    offsetAdjustment = Gtk::Adjustment::create(0.0, -2000.0, 5000.0, 10.0, 50.0, 0.0);
     offsetButton->configure(offsetAdjustment, 0, 2);
-#else
-    offsetAdjustment = new Gtk::Adjustment(0.0, -2000.0, 2000.0, 10.0, 50.0, 0.0);
-    offsetButton->configure(*offsetAdjustment, 0.0, 2);
-#endif  /* HAVE_GTK3 */
 }
 
 void DeviceWidget::init(MainWindow* mainWindow, Glib::ustring deviceType) {
     mpMainWindow = mainWindow;
     mDeviceType = deviceType;
+
+    MinimalStreamWidget::init();
 }
 
 void DeviceWidget::setChannelMap(const pa_channel_map &m, bool can_decibel) {
     channelMap = m;
+    ChannelWidget::create(this, m, can_decibel, channelWidgets);
 
     for (int i = 0; i < m.channels; i++) {
-        ChannelWidget *cw = channelWidgets[i] = ChannelWidget::create();
-        cw->channel = i;
-        cw->can_decibel = can_decibel;
-        cw->minimalStreamWidget = this;
-        char text[64];
-        snprintf(text, sizeof(text), "<b>%s</b>", pa_channel_position_to_pretty_string(m.map[i]));
-        cw->channelLabel->set_markup(text);
+        ChannelWidget *cw = channelWidgets[i];
         channelsVBox->pack_start(*cw, false, false, 0);
         cw->unreference();
     }
-    channelWidgets[m.channels-1]->last = true;
 
     lockToggleButton->set_sensitive(m.channels > 1);
     hideLockedChannels(lockToggleButton->get_active());
