@@ -37,7 +37,7 @@
 
 /* Used for profile sorting */
 struct profile_prio_compare {
-    bool operator() (const pa_card_profile_info& lhs, const pa_card_profile_info& rhs) const {
+    bool operator() (const pa_card_profile_info2& lhs, const pa_card_profile_info2& rhs) const {
 
         if (lhs.priority == rhs.priority)
             return strcmp(lhs.name, rhs.name) > 0;
@@ -93,6 +93,7 @@ MainWindow::MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
     x->get_widget("sinkTypeComboBox", sinkTypeComboBox);
     x->get_widget("sourceTypeComboBox", sourceTypeComboBox);
     x->get_widget("notebook", notebook);
+    x->get_widget("showVolumeMetersCheckButton", showVolumeMetersCheckButton);
 
     cardsVBox->set_reallocate_redraws(true);
     sourcesVBox->set_reallocate_redraws(true);
@@ -109,6 +110,8 @@ MainWindow::MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
     sourceOutputTypeComboBox->signal_changed().connect(sigc::mem_fun(*this, &MainWindow::onSourceOutputTypeComboBoxChanged));
     sinkTypeComboBox->signal_changed().connect(sigc::mem_fun(*this, &MainWindow::onSinkTypeComboBoxChanged));
     sourceTypeComboBox->signal_changed().connect(sigc::mem_fun(*this, &MainWindow::onSourceTypeComboBoxChanged));
+    showVolumeMetersCheckButton->signal_toggled().connect(sigc::mem_fun(*this, &MainWindow::onShowVolumeMetersCheckButtonToggled));
+
 
     GKeyFile* config = g_key_file_new();
     g_assert(config);
@@ -121,6 +124,13 @@ MainWindow::MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
         int width  = g_key_file_get_integer(config, "window", "width", NULL);
         int height = g_key_file_get_integer(config, "window", "height", NULL);
 
+        /* When upgrading from a previous version, set showVolumeMeters to TRUE
+         * (default from glade file), so users don't complain about missing
+         * volume meters. */
+        if (g_key_file_has_key(config, "window", "showVolumeMeters", NULL)) {
+            showVolumeMetersCheckButton->set_active(g_key_file_get_boolean(config, "window", "showVolumeMeters", NULL));
+        }
+
         int default_width, default_height;
         get_default_size(default_width, default_height);
         if (width >= default_width && height >= default_height)
@@ -186,19 +196,11 @@ MainWindow* MainWindow::create(bool maximize) {
 void MainWindow::on_realize() {
     Gtk::Window::on_realize();
 
-#ifdef HAVE_GTK3
     get_window()->set_cursor(Gdk::Cursor::create(Gdk::WATCH));
-#else
-    get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
-#endif /* HAVE_GTK3 */
 }
 
 bool MainWindow::on_key_press_event(GdkEventKey* event) {
 
-    if (GDK_KEY_Escape == event->keyval) {
-        Gtk::Main::quit();
-        return true;
-    }
     if (event->state & GDK_CONTROL_MASK) {
         switch (event->keyval) {
             case GDK_KEY_KP_1:
@@ -238,6 +240,7 @@ MainWindow::~MainWindow() {
     g_key_file_set_integer(config, "window", "sourceOutputType", sourceOutputTypeComboBox->get_active_row_number());
     g_key_file_set_integer(config, "window", "sinkType", sinkTypeComboBox->get_active_row_number());
     g_key_file_set_integer(config, "window", "sourceType", sourceTypeComboBox->get_active_row_number());
+    g_key_file_set_integer(config, "window", "showVolumeMeters", showVolumeMetersCheckButton->get_active());
 
     gsize filelen;
     GError *err = NULL;
@@ -279,7 +282,7 @@ static void set_icon_name_fallback(Gtk::Image *i, const char *name, Gtk::IconSiz
     theme = Gtk::IconTheme::get_default();
 
     try {
-        pixbuf = theme->load_icon(name, width, Gtk::ICON_LOOKUP_GENERIC_FALLBACK);
+        pixbuf = theme->load_icon(name, width, Gtk::ICON_LOOKUP_GENERIC_FALLBACK | Gtk::ICON_LOOKUP_FORCE_SIZE);
 
         if (pixbuf)
             i->set(pixbuf);
@@ -329,11 +332,27 @@ static void updatePorts(DeviceWidget *w, std::map<Glib::ustring, PortInfo> &port
     }
 }
 
+void MainWindow::selectBestTab() {
+    if (sinkInputWidgets.size() > 0)
+        notebook->set_current_page(0);
+    else if (sourceOutputWidgets.size() > 0)
+        notebook->set_current_page(1);
+    else if (sourceWidgets.size() > 0 && sinkWidgets.size() == 0)
+        notebook->set_current_page(3);
+    else
+        notebook->set_current_page(2);
+}
+
+void MainWindow::selectTab(int tab_number) {
+    if (tab_number > 0 && tab_number <= notebook->get_n_pages())
+        notebook->set_current_page(tab_number - 1);
+}
+
 void MainWindow::updateCard(const pa_card_info &info) {
     CardWidget *w;
     bool is_new = false;
     const char *description, *icon;
-    std::set<pa_card_profile_info,profile_prio_compare> profile_priorities;
+    std::set<pa_card_profile_info2,profile_prio_compare> profile_priorities;
 
     if (cardWidgets.count(info.index))
         w = cardWidgets[info.index];
@@ -359,7 +378,7 @@ void MainWindow::updateCard(const pa_card_info &info) {
     for (uint32_t i=0; i<info.n_profiles; ++i) {
         w->hasSinks = w->hasSinks || (info.profiles[i].n_sinks > 0);
         w->hasSources = w->hasSources || (info.profiles[i].n_sources > 0);
-        profile_priorities.insert(info.profiles[i]);
+        profile_priorities.insert(*info.profiles2[i]);
     }
 
     w->ports.clear();
@@ -379,7 +398,7 @@ void MainWindow::updateCard(const pa_card_info &info) {
     }
 
     w->profiles.clear();
-    for (std::set<pa_card_profile_info>::iterator profileIt = profile_priorities.begin(); profileIt != profile_priorities.end(); ++profileIt) {
+    for (std::set<pa_card_profile_info2>::iterator profileIt = profile_priorities.begin(); profileIt != profile_priorities.end(); ++profileIt) {
         bool hasNo = false, hasOther = false;
         std::map<Glib::ustring, PortInfo>::iterator portIt;
         Glib::ustring desc = profileIt->description;
@@ -400,6 +419,9 @@ void MainWindow::updateCard(const pa_card_info &info) {
         if (hasNo && !hasOther)
             desc += _(" (unplugged)");
 
+        if (!profileIt->available)
+            desc += _(" (unavailable)");
+
         w->profiles.push_back(std::pair<Glib::ustring,Glib::ustring>(profileIt->name, desc));
     }
 
@@ -463,6 +485,7 @@ bool MainWindow::updateSink(const pa_sink_info &info) {
         is_new = true;
 
         w->setBaseVolume(info.base_volume);
+        w->setVolumeMeterVisible(showVolumeMetersCheckButton->get_active());
     }
 
     w->updating = true;
@@ -585,7 +608,8 @@ pa_stream* MainWindow::createMonitorStreamForSource(uint32_t source_idx, uint32_
     pa_stream_set_suspended_callback(s, suspended_callback, this);
 
     flags = (pa_stream_flags_t) (PA_STREAM_DONT_MOVE | PA_STREAM_PEAK_DETECT | PA_STREAM_ADJUST_LATENCY |
-                                 (suspend ? PA_STREAM_DONT_INHIBIT_AUTO_SUSPEND : PA_STREAM_NOFLAGS));
+                                 (suspend ? PA_STREAM_DONT_INHIBIT_AUTO_SUSPEND : PA_STREAM_NOFLAGS) |
+                                 (!showVolumeMetersCheckButton->get_active() ? PA_STREAM_START_CORKED : PA_STREAM_NOFLAGS));
 
     if (pa_stream_connect_record(s, t, &attr, flags) < 0) {
         show_error(_("Failed to connect monitoring stream"));
@@ -625,9 +649,10 @@ void MainWindow::updateSource(const pa_source_info &info) {
         is_new = true;
 
         w->setBaseVolume(info.base_volume);
+        w->setVolumeMeterVisible(showVolumeMetersCheckButton->get_active());
 
         if (pa_context_get_server_protocol_version(get_context()) >= 13)
-            createMonitorStreamForSource(info.index, -1, !!(info.flags & PA_SOURCE_NETWORK));
+            w->peak = createMonitorStreamForSource(info.index, -1, !!(info.flags & PA_SOURCE_NETWORK));
     }
 
     w->updating = true;
@@ -741,6 +766,7 @@ void MainWindow::updateSinkInput(const pa_sink_input_info &info) {
         w->index = info.index;
         w->clientIndex = info.client;
         is_new = true;
+        w->setVolumeMeterVisible(showVolumeMetersCheckButton->get_active());
 
         if (pa_context_get_server_protocol_version(get_context()) >= 13)
             createMonitorStreamForSinkInput(w, info.sink);
@@ -799,6 +825,7 @@ void MainWindow::updateSourceOutput(const pa_source_output_info &info) {
         w->index = info.index;
         w->clientIndex = info.client;
         is_new = true;
+        w->setVolumeMeterVisible(showVolumeMetersCheckButton->get_active());
     }
 
     w->updating = true;
@@ -1274,3 +1301,46 @@ void MainWindow::onSourceOutputTypeComboBoxChanged() {
 
     updateDeviceVisibility();
 }
+
+
+void MainWindow::onShowVolumeMetersCheckButtonToggled() {
+    bool state = showVolumeMetersCheckButton->get_active();
+    pa_operation *o;
+
+    for (std::map<uint32_t, SinkWidget*>::iterator it = sinkWidgets.begin() ; it != sinkWidgets.end(); it++) {
+        SinkWidget *sw = it->second;
+        if (sw->peak) {
+            o = pa_stream_cork(sw->peak, (int)!state, NULL, NULL);
+            if (o)
+                pa_operation_unref(o);
+        }
+        sw->setVolumeMeterVisible(state);
+    }
+    for (std::map<uint32_t, SourceWidget*>::iterator it = sourceWidgets.begin() ; it != sourceWidgets.end(); it++) {
+        SourceWidget *sw = it->second;
+        if (sw->peak) {
+            o = pa_stream_cork(sw->peak, (int)!state, NULL, NULL);
+            if (o)
+                pa_operation_unref(o);
+        }
+        sw->setVolumeMeterVisible(state);
+    }
+    for (std::map<uint32_t, SinkInputWidget*>::iterator it = sinkInputWidgets.begin() ; it != sinkInputWidgets.end(); it++) {
+        SinkInputWidget *sw = it->second;
+        if (sw->peak) {
+            o = pa_stream_cork(sw->peak, (int)!state, NULL, NULL);
+            if (o)
+                pa_operation_unref(o);
+        }
+        sw->setVolumeMeterVisible(state);
+    }
+    for (std::map<uint32_t, SourceOutputWidget*>::iterator it = sourceOutputWidgets.begin() ; it != sourceOutputWidgets.end(); it++) {
+        SourceOutputWidget *sw = it->second;
+        if (sw->peak) {
+            o = pa_stream_cork(sw->peak, (int)!state, NULL, NULL);
+            if (o)
+                pa_operation_unref(o);
+        }
+        sw->setVolumeMeterVisible(state);
+    }
+}
