@@ -21,6 +21,8 @@
 #ifndef mainwindow_h
 #define mainwindow_h
 
+class MainWindow;
+
 #include "pavucontrol.h"
 #include <pulse/ext-stream-restore.h>
 #if HAVE_EXT_DEVICE_RESTORE_API
@@ -60,6 +62,9 @@ public:
     void removeSourceOutput(uint32_t index);
     void removeClient(uint32_t index);
 
+    void selectBestTab();
+    void selectTab(int tab_number);
+
     void removeAllWidgets();
 
     void setConnectingMessage(const char *string = NULL);
@@ -68,6 +73,7 @@ public:
     Gtk::VBox *streamsVBox, *recsVBox, *sinksVBox, *sourcesVBox, *cardsVBox;
     Gtk::Label *noStreamsLabel, *noRecsLabel, *noSinksLabel, *noSourcesLabel, *noCardsLabel, *connectingLabel;
     Gtk::ComboBox *sinkInputTypeComboBox, *sourceOutputTypeComboBox, *sinkTypeComboBox, *sourceTypeComboBox;
+    Gtk::CheckButton *showVolumeMetersCheckButton;
 
     std::map<uint32_t, CardWidget*> cardWidgets;
     std::map<uint32_t, SinkWidget*> sinkWidgets;
@@ -85,6 +91,7 @@ public:
     virtual void onSourceOutputTypeComboBoxChanged();
     virtual void onSinkTypeComboBoxChanged();
     virtual void onSourceTypeComboBoxChanged();
+    virtual void onShowVolumeMetersCheckButtonToggled();
 
     void setConnectionState(gboolean connected);
     void updateDeviceVisibility();
